@@ -1,7 +1,9 @@
 coef.permutest.rma.uni <- function(object, ...) {
 
+   mstyle <- .get.mstyle("crayon" %in% .packages())
+
    if (!inherits(object, "permutest.rma.uni"))
-      stop("Argument 'object' must be an object of class \"permutest.rma.uni\".")
+      stop(mstyle$stop("Argument 'object' must be an object of class \"permutest.rma.uni\"."))
 
    x <- object
 
