@@ -1,7 +1,9 @@
 confint.rma.mv <- function(object, parm, level, fixed=FALSE, sigma2, tau2, rho, gamma2, phi, digits, transf, targs, verbose=FALSE, control, ...) {
 
+   mstyle <- .get.mstyle("crayon" %in% .packages())
+
    if (!inherits(object, "rma.mv"))
-      stop("Argument 'object' must be an object of class \"rma.mv\".")
+      stop(mstyle$stop("Argument 'object' must be an object of class \"rma.mv\"."))
 
    x <- object
 
@@ -11,8 +13,11 @@ confint.rma.mv <- function(object, parm, level, fixed=FALSE, sigma2, tau2, rho,
    if (missing(level))
       level <- x$level
 
-   if (missing(digits))
-      digits <- x$digits
+   if (missing(digits)) {
+      digits <- .get.digits(xdigits=x$digits, dmiss=TRUE)
+   } else {
+      digits <- .get.digits(digits=digits, xdigits=x$digits, dmiss=FALSE)
+   }
 
    if (missing(transf))
       transf <- FALSE
@@ -23,7 +28,7 @@ confint.rma.mv <- function(object, parm, level, fixed=FALSE, sigma2, tau2, rho,
    if (missing(control))
       control <- list()
 
-   level <- ifelse(level > 1, (100-level)/100, ifelse(level > .5, 1-level, level))
+   level <- ifelse(level == 0, 1, ifelse(level >= 1, (100-level)/100, ifelse(level > .5, 1-level, level)))
 
    ### check if user has specified one of the sigma2, tau2, rho, gamma2, or phi arguments
 
@@ -42,7 +47,7 @@ confint.rma.mv <- function(object, parm, level, fixed=FALSE, sigma2, tau2, rho,
                ifelse(x$withH, sum(!x$vc.fix$gamma2) + sum(!x$vc.fix$phi), 0)
 
       if (comps == 0)
-         stop("No components for which a CI can be obtained.")
+         stop(mstyle$stop("No components for which a CI can be obtained."))
 
       res.all <- list()
       j <- 0
@@ -54,7 +59,7 @@ confint.rma.mv <- function(object, parm, level, fixed=FALSE, sigma2, tau2, rho,
             cl.vc$sigma2 <- pos
             cl.vc$object <- quote(x)
             if (verbose)
-               cat("\nObtaining CI for sigma2 =", pos, "\n")
+               cat(mstyle$verbose(paste("\nObtaining CI for sigma2 =", pos, "\n")))
             res.all[[j]] <- eval(cl.vc)
          }
       }
@@ -67,7 +72,7 @@ confint.rma.mv <- function(object, parm, level, fixed=FALSE, sigma2, tau2, rho,
                cl.vc$tau2 <- pos
                cl.vc$object <- quote(x)
                if (verbose)
-                  cat("\nObtaining CI for tau2 =", pos, "\n")
+                  cat(mstyle$verbose(paste("\nObtaining CI for tau2 =", pos, "\n")))
                res.all[[j]] <- eval(cl.vc)
             }
          }
@@ -78,7 +83,7 @@ confint.rma.mv <- function(object, parm, level, fixed=FALSE, sigma2, tau2, rho,
                cl.vc$rho <- pos
                cl.vc$object <- quote(x)
                if (verbose)
-                  cat("\nObtaining CI for rho =", pos, "\n")
+                  cat(mstyle$verbose(paste("\nObtaining CI for rho =", pos, "\n")))
                res.all[[j]] <- eval(cl.vc)
             }
          }
@@ -92,7 +97,7 @@ confint.rma.mv <- function(object, parm, level, fixed=FALSE, sigma2, tau2, rho,
                cl.vc$gamma2 <- pos
                cl.vc$object <- quote(x)
                if (verbose)
-                  cat("\nObtaining CI for gamma2 =", pos, "\n")
+                  cat(mstyle$verbose(paste("\nObtaining CI for gamma2 =", pos, "\n")))
                res.all[[j]] <- eval(cl.vc)
             }
          }
@@ -103,7 +108,7 @@ confint.rma.mv <- function(object, parm, level, fixed=FALSE, sigma2, tau2, rho,
                cl.vc$phi <- pos
                cl.vc$object <- quote(x)
                if (verbose)
-                  cat("\nObtaining CI for phi =", pos, "\n")
+                  cat(mstyle$verbose(paste("\nObtaining CI for phi =", pos, "\n")))
                res.all[[j]] <- eval(cl.vc)
             }
          }
@@ -112,6 +117,8 @@ confint.rma.mv <- function(object, parm, level, fixed=FALSE, sigma2, tau2, rho,
       if (length(res.all) == 1) {
          return(res.all[[1]])
       } else {
+         res.all$digits <- digits
+         class(res.all) <- "list.confint.rma"
          return(res.all)
       }
 
@@ -130,93 +137,93 @@ confint.rma.mv <- function(object, parm, level, fixed=FALSE, sigma2, tau2, rho,
       ### check if user has specified more than one of these arguments
 
       if (sum(!missing(sigma2), !missing(tau2), !missing(rho), !missing(gamma2), !missing(phi)) > 1L)
-         stop("Must specify only one of the arguments 'sigma2', 'tau2', 'rho', 'gamma2', or 'phi'.")
+         stop(mstyle$stop("Must specify only one of the arguments 'sigma2', 'tau2', 'rho', 'gamma2', or 'phi'."))
 
       ### check if model actually contains (at least one) such a component and that it was actually estimated
       ### note: a component that is not in the model is NA; components that are fixed are TRUE
 
       if (!missing(sigma2) && (all(is.na(x$vc.fix$sigma2)) || all(x$vc.fix$sigma2)))
-         stop("Model does not contain any (estimated) 'sigma2' components.")
+         stop(mstyle$stop("Model does not contain any (estimated) 'sigma2' components."))
 
       if (!missing(tau2) && (all(is.na(x$vc.fix$tau2)) || all(x$vc.fix$tau2)))
-         stop("Model does not contain any (estimated) 'tau2' components.")
+         stop(mstyle$stop("Model does not contain any (estimated) 'tau2' components."))
 
       if (!missing(rho) && c(all(is.na(x$vc.fix$rho)) || all(x$vc.fix$rho)))
-         stop("Model does not contain any (estimated) 'rho' components.")
+         stop(mstyle$stop("Model does not contain any (estimated) 'rho' components."))
 
       if (!missing(gamma2) && (all(is.na(x$vc.fix$gamma2)) || all(x$vc.fix$gamma2)))
-         stop("Model does not contain any (estimated) 'gamma2' components.")
+         stop(mstyle$stop("Model does not contain any (estimated) 'gamma2' components."))
 
       if (!missing(phi) && c(all(is.na(x$vc.fix$phi)) || all(x$vc.fix$phi)))
-         stop("Model does not contain any (estimated) 'phi' components.")
+         stop(mstyle$stop("Model does not contain any (estimated) 'phi' components."))
 
       ### check if user specified more than one sigma2, tau2, rho, gamma2, or rho component
 
       if (!missing(sigma2) && (length(sigma2) > 1L))
-         stop("Can only specify one 'sigma2' component.")
+         stop(mstyle$stop("Can only specify one 'sigma2' component."))
 
       if (!missing(tau2) && (length(tau2) > 1L))
-         stop("Can only specify one 'tau2' component.")
+         stop(mstyle$stop("Can only specify one 'tau2' component."))
 
       if (!missing(rho) && (length(rho) > 1L))
-         stop("Can only specify one 'rho' component.")
+         stop(mstyle$stop("Can only specify one 'rho' component."))
 
       if (!missing(gamma2) && (length(gamma2) > 1L))
-         stop("Can only specify one 'gamma2' component.")
+         stop(mstyle$stop("Can only specify one 'gamma2' component."))
 
       if (!missing(phi) && (length(phi) > 1L))
-         stop("Can only specify one 'phi' component.")
+         stop(mstyle$stop("Can only specify one 'phi' component."))
 
       ### check if user specified a logical
 
       if (!missing(sigma2) && is.logical(sigma2))
-         stop("Must specify the number for the 'sigma2' component.")
+         stop(mstyle$stop("Must specify the number for the 'sigma2' component."))
 
       if (!missing(tau2) && is.logical(tau2))
-         stop("Must specify the number for the 'tau2' component.")
+         stop(mstyle$stop("Must specify the number for the 'tau2' component."))
 
       if (!missing(rho) && is.logical(rho))
-         stop("Must specify the number for the 'rho' component.")
+         stop(mstyle$stop("Must specify the number for the 'rho' component."))
 
       if (!missing(gamma2) && is.logical(gamma2))
-         stop("Must specify the number for the 'gamma2' component.")
+         stop(mstyle$stop("Must specify the number for the 'gamma2' component."))
 
       if (!missing(phi) && is.logical(phi))
-         stop("Must specify the number for the 'phi' component.")
+         stop(mstyle$stop("Must specify the number for the 'phi' component."))
 
       ### check if user specified a component that does not exist
 
       if (!missing(sigma2) && (sigma2 > length(x$vc.fix$sigma2) || sigma2 <= 0))
-         stop("No such 'sigma2' component in the model.")
+         stop(mstyle$stop("No such 'sigma2' component in the model."))
 
       if (!missing(tau2) && (tau2 > length(x$vc.fix$tau2) || tau2 <= 0))
-         stop("No such 'tau2' component in the model.")
+         stop(mstyle$stop("No such 'tau2' component in the model."))
 
       if (!missing(rho) && (rho > length(x$vc.fix$rho) || rho <= 0))
-         stop("No such 'rho' component in the model.")
+         stop(mstyle$stop("No such 'rho' component in the model."))
 
       if (!missing(gamma2) && (gamma2 > length(x$vc.fix$gamma2) || gamma2 <= 0))
-         stop("No such 'gamma2' component in the model.")
+         stop(mstyle$stop("No such 'gamma2' component in the model."))
 
       if (!missing(phi) && (phi > length(x$vc.fix$phi) || phi <= 0))
-         stop("No such 'phi' component in the model.")
+         stop(mstyle$stop("No such 'phi' component in the model."))
 
       ### check if user specified a component that was fixed
 
       if (!missing(sigma2) && x$vc.fix$sigma2[sigma2])
-         stop("Specified 'sigma2' component was fixed.")
+         stop(mstyle$stop("Specified 'sigma2' component was fixed."))
 
       if (!missing(tau2) && x$vc.fix$tau2[tau2])
-         stop("Specified 'tau2' component was fixed.")
+         stop(mstyle$stop("Specified 'tau2' component was fixed."))
 
       if (!missing(rho) && x$vc.fix$rho[rho])
-         stop("Specified 'rho' component was fixed.")
+         stop(mstyle$stop("Specified 'rho' component was fixed."))
 
       if (!missing(gamma2) && x$vc.fix$gamma2[gamma2])
-         stop("Specified 'gamma2' component was fixed.")
+         stop(mstyle$stop("Specified 'gamma2' component was fixed."))
 
       if (!missing(phi) && x$vc.fix$phi[phi])
-         stop("Specified 'phi' component was fixed.")
+         stop(mstyle$stop("Specified 'phi' component was fixed."))
 
       ### if everything is good so far, get value of the variance component and set 'comp'
 
@@ -270,24 +277,40 @@ confint.rma.mv <- function(object, parm, level, fixed=FALSE, sigma2, tau2, rho,
          con$vc.max <- max(ifelse(vc <= .Machine$double.eps^0.5, 10, max(10, vc*100)), con$vc.min)
       }
       if (comp == "rho") {
-         if (is.element(x$struct[1], c("CS", "HCS")))
+         if (is.element(x$struct[1], c("CS","HCS")))
             con$vc.min <- -1                                  ### this will fail most of the time but with retries, this may get closer to actual lower bound
             #con$vc.min <- min(-1/(x$g.nlevels.f[1] - 1), vc) ### this guarantees that cor matrix is semi-positive definite, but since V gets added, this is actually too strict
-         if (is.element(x$struct[1], c("AR", "HAR")))
-            con$vc.min <- min(0, vc)                          ### negative autocorrelation parameters not considered
-         if (is.element(x$struct[1], c("UN", "UNHO")))
+         if (is.element(x$struct[1], c("AR","HAR","CAR")))
+            con$vc.min <- min(0, vc)                          ### negative autocorrelation parameters not considered (not even sensible for CAR)
+         if (is.element(x$struct[1], c("UN","UNHO","GEN")))
             con$vc.min <- -1                                  ### TODO: this will often fail! (but with retries, this should still work)
          con$vc.max <- 1
+         if (is.element(x$struct[1], c("SPEXP","SPGAU","SPLIN","SPRAT","SPSPH"))) {
+            con$vc.min <- 0                                   ### TODO: 0 basically always fails
+            con$vc.max <- max(10, vc*10)
+         }
+         if (is.element(x$struct[1], c("PHYPL","PHYPD"))) {
+            con$vc.min <- 0
+            con$vc.max <- max(2, vc*2)
+         }
       }
       if (comp == "phi") {
-         if (is.element(x$struct[2], c("CS", "HCS")))
+         if (is.element(x$struct[2], c("CS","HCS")))
             con$vc.min <- -1                                  ### this will fail most of the time but with retries, this may get closer to actual lower bound
             #con$vc.min <- min(-1/(x$h.nlevels.f[1] - 1), vc) ### this guarantees that cor matrix is semi-positive definite, but since V gets added, this is actually too strict
-         if (is.element(x$struct[2], c("AR", "HAR")))
-            con$vc.min <- min(0, vc)                          ### negative autocorrelation parameters not considered
-         if (is.element(x$struct[2], c("UN", "UNHO")))
+         if (is.element(x$struct[2], c("AR","HAR","CAR")))
+            con$vc.min <- min(0, vc)                          ### negative autocorrelation parameters not considered (not even sensible for CAR)
+         if (is.element(x$struct[2], c("UN","UNHO","GEN")))
             con$vc.min <- -1                                  ### TODO: this will often fail! (but with retries, this should still work)
          con$vc.max <- 1
+         if (is.element(x$struct[2], c("SPEXP","SPGAU","SPLIN","SPRAT","SPSPH"))) {
+            con$vc.min <- 0                                   ### TODO: 0 basically always fails
+            con$vc.max <- max(10, vc*10)
+         }
+         if (is.element(x$struct[2], c("PHYPL","PHYPD"))) {
+            con$vc.min <- 0
+            con$vc.max <- max(2, vc*2)
+         }
       }
 
       con[pmatch(names(control), names(con))] <- control
@@ -304,8 +327,8 @@ confint.rma.mv <- function(object, parm, level, fixed=FALSE, sigma2, tau2, rho,
       ci.null <- FALSE ### logical if CI is a null set
       lb.conv <- FALSE ### logical if search converged for lower bound (LB)
       ub.conv <- FALSE ### logical if search converged for upper bound (UB)
-      lb.sign <- ""    ### for sign in case LB must be below tau2.min ("<") or above tau2.max (">")
-      ub.sign <- ""    ### for sign in case UB must be below tau2.min ("<") or above tau2.max (">")
+      lb.sign <- ""    ### for sign in case LB must be below vc.min ("<") or above vc.max (">")
+      ub.sign <- ""    ### for sign in case UB must be below vc.min ("<") or above vc.max (">")
 
       ######################################################################
       ######################################################################
@@ -316,9 +339,9 @@ confint.rma.mv <- function(object, parm, level, fixed=FALSE, sigma2, tau2, rho,
       if (type == "PL") {
 
          if (con$vc.min > vc)
-            stop("Lower bound of interval to be searched must be <= estimated value of component.")
+            stop(mstyle$stop("Lower bound of interval to be searched must be <= estimated value of component."))
          if (con$vc.max < vc)
-            stop("Upper bound of interval to be searched must be >= estimated value of component.")
+            stop(mstyle$stop("Upper bound of interval to be searched must be >= estimated value of component."))
 
          objective <- qchisq(1-level, df=1)
 
@@ -329,13 +352,10 @@ confint.rma.mv <- function(object, parm, level, fixed=FALSE, sigma2, tau2, rho,
          ### get diff value when setting component to vc.min; this value should be positive (i.e., discrepancy must be larger than critical value)
          ### if it is not, then the lower bound must be below vc.min
 
-         epdiff <- seq(0, abs(con$vc.min - vc), length=con$eptries+1) ### e.g., if vc.min=0, vc=.5, and eptries=10, then get 0, .05, .10, ..., .50
-         epdiff <- epdiff[-(con$eptries+1)]                           ### this then strips the last entry (.50), so we get 0, .05, .10, ..., .45
+         epdiff <- abs(con$vc.min - vc) / con$eptries
 
          for (i in seq_len(con$eptries)) {
 
-            con$vc.min <- con$vc.min + epdiff[i]
-
             res <- try(.profile.rma.mv(val = con$vc.min, obj=x, comp=comp, sigma2.pos=sigma2.pos, tau2.pos=tau2.pos, rho.pos=rho.pos, gamma2.pos=gamma2.pos, phi.pos=phi.pos, CI=TRUE, objective=objective, verbose=verbose), silent=TRUE)
 
             if (!inherits(res, "try-error")) {
@@ -351,6 +371,9 @@ confint.rma.mv <- function(object, parm, level, fixed=FALSE, sigma2, tau2, rho,
                   if (is.element(comp, c("rho", "phi")) && con$vc.min > -1)
                      lb.sign <- "<"
 
+                  if (((comp == "rho" && is.element(x$struct[1], c("SPEXP","SPGAU","SPLIN","SPRAT","SPSPH","PHYPL","PHYPD"))) || (comp == "phi" && is.element(x$struct[2], c("SPEXP","SPGAU","SPLIN","SPRAT","SPSPH","PHYPL","PHYPD")))) && con$vc.min > 0)
+                     lb.sign <- "<"
+
                } else {
 
                   res <- try(uniroot(.profile.rma.mv, interval=c(con$vc.min, vc), tol=con$tol, maxiter=con$maxiter, obj=x, comp=comp, sigma2.pos=sigma2.pos, tau2.pos=tau2.pos, rho.pos=rho.pos, gamma2.pos=gamma2.pos, phi.pos=phi.pos, CI=TRUE, objective=objective, verbose=verbose, check.conv=TRUE)$root, silent=TRUE)
@@ -367,6 +390,8 @@ confint.rma.mv <- function(object, parm, level, fixed=FALSE, sigma2, tau2, rho,
 
             }
 
+            con$vc.min <- con$vc.min + epdiff
+
          }
 
          if (verbose)
@@ -379,13 +404,10 @@ confint.rma.mv <- function(object, parm, level, fixed=FALSE, sigma2, tau2, rho,
          ### get diff value when setting component to vc.max; this value should be positive (i.e., discrepancy must be larger than critical value)
          ### if it is not, then the upper bound must be above vc.max
 
-         epdiff <- seq(0, abs(con$vc.max - vc), length=con$eptries+1) ### e.g., if vc.max=1, vc=.5, and eptries=10, then get 0, .05, .10, ..., .50
-         epdiff <- epdiff[-(con$eptries+1)]                           ### this then strips the last entry (.50), so we get 0, .05, .10, ..., .45
+         epdiff <- abs(con$vc.max - vc) / con$eptries
 
          for (i in seq_len(con$eptries)) {
 
-            con$vc.max <- con$vc.max - epdiff[i]
-
             res <- try(.profile.rma.mv(val = con$vc.max, obj=x, comp=comp, sigma2.pos=sigma2.pos, tau2.pos=tau2.pos, rho.pos=rho.pos, gamma2.pos=gamma2.pos, phi.pos=phi.pos, CI=TRUE, objective=objective, verbose=verbose), silent=TRUE)
 
             if (!inherits(res, "try-error")) {
@@ -401,6 +423,9 @@ confint.rma.mv <- function(object, parm, level, fixed=FALSE, sigma2, tau2, rho,
                   if (is.element(comp, c("rho", "phi")) && con$vc.max < 1)
                      ub.sign <- ">"
 
+                  if ((comp == "rho" && is.element(x$struct[1], c("SPEXP","SPGAU","SPLIN","SPRAT","SPSPH","PHYPL","PHYPD"))) || (comp == "phi" && is.element(x$struct[2], c("SPEXP","SPGAU","SPLIN","SPRAT","SPSPH","PHYPL","PHYPD"))))
+                     ub.sign <- ">"
+
                } else {
 
                   res <- try(uniroot(.profile.rma.mv, interval=c(vc, con$vc.max), tol=con$tol, maxiter=con$maxiter, obj=x, comp=comp, sigma2.pos=sigma2.pos, tau2.pos=tau2.pos, rho.pos=rho.pos, gamma2.pos=gamma2.pos, phi.pos=phi.pos, CI=TRUE, objective=objective, verbose=verbose, check.conv=TRUE)$root, silent=TRUE)
@@ -417,6 +442,8 @@ confint.rma.mv <- function(object, parm, level, fixed=FALSE, sigma2, tau2, rho,
 
             }
 
+            con$vc.max <- con$vc.max - epdiff
+
          }
 
          ###################################################################
@@ -428,10 +455,10 @@ confint.rma.mv <- function(object, parm, level, fixed=FALSE, sigma2, tau2, rho,
       ######################################################################
 
       if (!lb.conv)
-         warning("Cannot obtain lower bound of profile likelihood CI due to convergence problems.")
+         warning(mstyle$warning("Cannot obtain lower bound of profile likelihood CI due to convergence problems."))
 
       if (!ub.conv)
-         warning("Cannot obtain upper bound of profile likelihood CI due to convergence problems.")
+         warning(mstyle$warning("Cannot obtain upper bound of profile likelihood CI due to convergence problems."))
 
       ######################################################################
 
