@@ -1,15 +1,20 @@
 confint.rma.peto <- function(object, parm, level, digits, transf, targs, ...) {
 
+   mstyle <- .get.mstyle("crayon" %in% .packages())
+
    if (!inherits(object, "rma.peto"))
-      stop("Argument 'object' must be an object of class \"rma.peto\".")
+      stop(mstyle$stop("Argument 'object' must be an object of class \"rma.peto\"."))
 
    x <- object
 
    if (missing(level))
       level <- x$level
 
-   if (missing(digits))
-      digits <- x$digits
+   if (missing(digits)) {
+      digits <- .get.digits(xdigits=x$digits, dmiss=TRUE)
+   } else {
+      digits <- .get.digits(digits=digits, xdigits=x$digits, dmiss=FALSE)
+   }
 
    if (missing(transf))
       transf <- FALSE
@@ -19,7 +24,7 @@ confint.rma.peto <- function(object, parm, level, digits, transf, targs, ...) {
 
    #########################################################################
 
-   level <- ifelse(level > 1, (100-level)/100, ifelse(level > .5, 1-level, level))
+   level <- ifelse(level == 0, 1, ifelse(level >= 1, (100-level)/100, ifelse(level > .5, 1-level, level)))
    crit  <- qnorm(level/2, lower.tail=FALSE)
 
    beta  <- x$beta
@@ -28,7 +33,7 @@ confint.rma.peto <- function(object, parm, level, digits, transf, targs, ...) {
 
    ### if requested, apply transformation function
 
-   if (is.logical(transf) && transf) ### if transf=TRUE, apply exp transformation to ORs
+   if (.isTRUE(transf)) ### if transf=TRUE, apply exp transformation to ORs
       transf <- exp
 
    if (is.function(transf)) {
