@@ -1,7 +1,9 @@
 deviance.rma <- function(object, REML, ...) {
 
+   mstyle <- .get.mstyle("crayon" %in% .packages())
+
    if (!inherits(object, "rma"))
-      stop("Argument 'object' must be an object of class \"rma\".")
+      stop(mstyle$stop("Argument 'object' must be an object of class \"rma\"."))
 
    if (missing(REML)) {
       if (object$method == "REML") {
