@@ -1,25 +1,29 @@
-permutest.rma.uni <- function(x, exact=FALSE, iter=1000, permci=FALSE, progbar=TRUE, retpermdist=FALSE, digits, tol, control, ...) {
+permutest.rma.uni <- function(x, exact=FALSE, iter=1000, permci=FALSE, progbar=TRUE, retpermdist=FALSE, digits, control, ...) {
+
+   mstyle <- .get.mstyle("crayon" %in% .packages())
 
    if (!inherits(x, "rma.uni"))
-      stop("Argument 'x' must be an object of class \"rma.uni\".")
+      stop(mstyle$stop("Argument 'x' must be an object of class \"rma.uni\"."))
 
    if (inherits(x, "robust.rma"))
-      stop("Method not yet implemented for objects of class \"robust.rma\". Sorry!")
+      stop(mstyle$stop("Method not available for objects of class \"robust.rma\"."))
 
    if (inherits(x, "rma.ls"))
-      stop("Method not yet implemented for objects of class \"rma.ls\". Sorry!")
+      stop(mstyle$stop("Method not available for objects of class \"rma.ls\"."))
 
-   if (missing(digits))
-      digits <- x$digits
-
-   ### default numerical tolerance
-
-   if (missing(tol)) {
-      comp.tol <- .Machine$double.eps^0.5
+   if (missing(digits)) {
+      digits <- .get.digits(xdigits=x$digits, dmiss=TRUE)
    } else {
-      comp.tol <- tol
+      digits <- .get.digits(digits=digits, xdigits=x$digits, dmiss=FALSE)
    }
 
+   ddd <- list(...)
+
+   .chkdots(ddd, c("tol", "seed"))
+
+   if (!is.null(ddd$tol)) # in case user specifies comptol in the old manner
+      comptol <- ddd$tol
+
    #########################################################################
 
    ### calculate number of permutations for an exact permutation test
@@ -71,7 +75,7 @@ permutest.rma.uni <- function(x, exact=FALSE, iter=1000, permci=FALSE, progbar=T
    }
 
    if (iter == Inf)
-      stop("Too many iterations required for exact permutation test.\n")
+      stop(mstyle$stop("Too many iterations required for exact permutation test."))
 
    #########################################################################
 
@@ -88,16 +92,24 @@ permutest.rma.uni <- function(x, exact=FALSE, iter=1000, permci=FALSE, progbar=T
    if (missing(control))
       control <- list()
 
-   con <- list(tol=.Machine$double.eps^0.25, maxiter=100, alternative="two.sided", p2defn="abs", stat="test", cialt="one.sided", seed=seed, distfac=1)
+   con <- list(comptol=.Machine$double.eps^0.5, tol=.Machine$double.eps^0.25, maxiter=100, alternative="two.sided", p2defn="abs", stat="test", cialt="one.sided", seed=seed, distfac=1)
    con[pmatch(names(control), names(con))] <- control
 
-   if (!exact)
-      set.seed(con$seed)
+   if (exists("comptol", inherits=FALSE))
+      con$comptol <- comptol
+
+   if (!exact) {
+      if (!is.null(ddd$seed)) {
+         set.seed(ddd$seed)
+      } else {
+         set.seed(con$seed)
+      }
+   }
 
    #########################################################################
 
    if (progbar)
-      cat("Running ", iter, " iterations for ", ifelse(exact, "exact", "approximate"), " permutation test.\n", sep="")
+      cat(mstyle$verbose(paste0("Running ", iter, " iterations for ", ifelse(exact, "exact", "approximate"), " permutation test.\n")))
 
    if (x$int.only) {
 
@@ -106,17 +118,17 @@ permutest.rma.uni <- function(x, exact=FALSE, iter=1000, permci=FALSE, progbar=T
       zval.perm <- try(rep(NA_real_, iter), silent=TRUE)
 
       if (inherits(zval.perm, "try-error"))
-         stop("Number of iterations requested too large.")
+         stop(mstyle$stop("Number of iterations requested too large."))
 
       beta.perm <- try(rep(NA_real_, iter), silent=TRUE)
 
       if (inherits(beta.perm, "try-error"))
-         stop("Number of iterations requested too large.")
+         stop(mstyle$stop("Number of iterations requested too large."))
 
       QM.perm <- try(rep(NA_real_, iter), silent=TRUE)
 
       if (inherits(QM.perm, "try-error"))
-         stop("Number of iterations requested too large.")
+         stop(mstyle$stop("Number of iterations requested too large."))
 
       if (progbar)
          pbar <- txtProgressBar(min=0, max=iter, style=3)
@@ -185,9 +197,9 @@ permutest.rma.uni <- function(x, exact=FALSE, iter=1000, permci=FALSE, progbar=T
             ### absolute value definition of the two-sided p-value
 
             if (con$stat == "test") {
-               pval <- mean(abs(zval.perm) >= abs(x$zval) - comp.tol, na.rm=TRUE) ### based on test statistic
+               pval <- mean(abs(zval.perm) >= abs(x$zval) - con$comptol, na.rm=TRUE) ### based on test statistic
             } else {
-               pval <- mean(abs(beta.perm) >= abs(c(x$beta)) - comp.tol, na.rm=TRUE) ### based on coefficient
+               pval <- mean(abs(beta.perm) >= abs(c(x$beta)) - con$comptol, na.rm=TRUE) ### based on coefficient
             }
 
          } else {
@@ -196,15 +208,15 @@ permutest.rma.uni <- function(x, exact=FALSE, iter=1000, permci=FALSE, progbar=T
 
             if (con$stat == "test") {
                if (x$zval > 0) {
-                  pval <- 2*mean(zval.perm >= x$zval - comp.tol, na.rm=TRUE) ### based on test statistic
+                  pval <- 2*mean(zval.perm >= x$zval - con$comptol, na.rm=TRUE) ### based on test statistic
                } else {
-                  pval <- 2*mean(zval.perm <= x$zval + comp.tol, na.rm=TRUE) ### based on coefficient
+                  pval <- 2*mean(zval.perm <= x$zval + con$comptol, na.rm=TRUE) ### based on coefficient
                }
             } else {
                if (c(x$beta) > 0) {
-                  pval <- 2*mean(beta.perm >= c(x$beta) - comp.tol, na.rm=TRUE) ### based on test statistic
+                  pval <- 2*mean(beta.perm >= c(x$beta) - con$comptol, na.rm=TRUE) ### based on test statistic
                } else {
-                  pval <- 2*mean(beta.perm <= c(x$beta) + comp.tol, na.rm=TRUE) ### based on coefficient
+                  pval <- 2*mean(beta.perm <= c(x$beta) + con$comptol, na.rm=TRUE) ### based on coefficient
                }
             }
 
@@ -214,23 +226,23 @@ permutest.rma.uni <- function(x, exact=FALSE, iter=1000, permci=FALSE, progbar=T
 
       if (con$alternative == "less") {
          if (con$stat == "test") {
-            pval <- mean(zval.perm <= x$zval + comp.tol, na.rm=TRUE) ### based on test statistic
+            pval <- mean(zval.perm <= x$zval + con$comptol, na.rm=TRUE) ### based on test statistic
          } else {
-            pval <- mean(beta.perm <= c(x$beta) + comp.tol, na.rm=TRUE) ### based on coefficient
+            pval <- mean(beta.perm <= c(x$beta) + con$comptol, na.rm=TRUE) ### based on coefficient
          }
       }
 
       if (con$alternative == "greater") {
          if (con$stat == "test") {
-            pval <- mean(zval.perm >= x$zval - comp.tol, na.rm=TRUE) ### based on test statistic
+            pval <- mean(zval.perm >= x$zval - con$comptol, na.rm=TRUE) ### based on test statistic
          } else {
-            pval <- mean(beta.perm >= c(x$beta) - comp.tol, na.rm=TRUE) ### based on coefficient
+            pval <- mean(beta.perm >= c(x$beta) - con$comptol, na.rm=TRUE) ### based on coefficient
          }
       }
 
       pval[pval > 1] <- 1
 
-      QMp <- mean(QM.perm >= x$QM - comp.tol, na.rm=TRUE)
+      QMp <- mean(QM.perm >= x$QM - con$comptol, na.rm=TRUE)
 
    #########################################################################
 
@@ -241,17 +253,17 @@ permutest.rma.uni <- function(x, exact=FALSE, iter=1000, permci=FALSE, progbar=T
       zval.perm <- try(suppressWarnings(matrix(NA_real_, nrow=iter, ncol=x$p)), silent=TRUE)
 
       if (inherits(zval.perm, "try-error"))
-         stop("Number of iterations requested too large.")
+         stop(mstyle$stop("Number of iterations requested too large."))
 
       beta.perm <- try(suppressWarnings(matrix(NA_real_, nrow=iter, ncol=x$p)), silent=TRUE)
 
       if (inherits(beta.perm, "try-error"))
-         stop("Number of iterations requested too large.")
+         stop(mstyle$stop("Number of iterations requested too large."))
 
       QM.perm <- try(rep(NA_real_, iter), silent=TRUE)
 
       if (inherits(QM.perm, "try-error"))
-         stop("Number of iterations requested too large.")
+         stop(mstyle$stop("Number of iterations requested too large."))
 
       if (progbar)
          pbar <- txtProgressBar(min=0, max=iter, style=3)
@@ -316,9 +328,9 @@ permutest.rma.uni <- function(x, exact=FALSE, iter=1000, permci=FALSE, progbar=T
             ### absolute value definition of the two-sided p-value
 
             if (con$stat == "test") {
-               pval <- rowMeans(t(abs(zval.perm)) >= abs(x$zval) - comp.tol, na.rm=TRUE) ### based on test statistics
+               pval <- rowMeans(t(abs(zval.perm)) >= abs(x$zval) - con$comptol, na.rm=TRUE) ### based on test statistics
             } else {
-               pval <- rowMeans(t(abs(beta.perm)) >= abs(c(x$beta)) - comp.tol, na.rm=TRUE) ### based on coefficients
+               pval <- rowMeans(t(abs(beta.perm)) >= abs(c(x$beta)) - con$comptol, na.rm=TRUE) ### based on coefficients
             }
 
          } else {
@@ -330,17 +342,17 @@ permutest.rma.uni <- function(x, exact=FALSE, iter=1000, permci=FALSE, progbar=T
             if (con$stat == "test") {
                for (j in seq_len(x$p)) {
                   if (x$zval[j] > 0) {
-                     pval[j] <- 2*mean(zval.perm[,j] >= x$zval[j] - comp.tol, na.rm=TRUE)
+                     pval[j] <- 2*mean(zval.perm[,j] >= x$zval[j] - con$comptol, na.rm=TRUE)
                   } else {
-                     pval[j] <- 2*mean(zval.perm[,j] <= x$zval[j] + comp.tol, na.rm=TRUE)
+                     pval[j] <- 2*mean(zval.perm[,j] <= x$zval[j] + con$comptol, na.rm=TRUE)
                   }
                }
             } else {
                for (j in seq_len(x$p)) {
                   if (c(x$beta)[j] > 0) {
-                     pval[j] <- 2*mean(beta.perm[,j] >= c(x$beta)[j] - comp.tol, na.rm=TRUE)
+                     pval[j] <- 2*mean(beta.perm[,j] >= c(x$beta)[j] - con$comptol, na.rm=TRUE)
                   } else {
-                     pval[j] <- 2*mean(beta.perm[,j] <= c(x$beta)[j] + comp.tol, na.rm=TRUE)
+                     pval[j] <- 2*mean(beta.perm[,j] <= c(x$beta)[j] + con$comptol, na.rm=TRUE)
                   }
                }
             }
@@ -351,23 +363,23 @@ permutest.rma.uni <- function(x, exact=FALSE, iter=1000, permci=FALSE, progbar=T
 
       if (con$alternative == "less") {
          if (con$stat == "test") {
-            pval <- rowMeans(t(zval.perm) <= x$zval + comp.tol, na.rm=TRUE) ### based on test statistics
+            pval <- rowMeans(t(zval.perm) <= x$zval + con$comptol, na.rm=TRUE) ### based on test statistics
          } else {
-            pval <- rowMeans(t(beta.perm) <= c(x$beta) + comp.tol, na.rm=TRUE) ### based on coefficients
+            pval <- rowMeans(t(beta.perm) <= c(x$beta) + con$comptol, na.rm=TRUE) ### based on coefficients
          }
       }
 
       if (con$alternative == "greater") {
          if (con$stat == "test") {
-            pval <- rowMeans(t(zval.perm) >= x$zval - comp.tol, na.rm=TRUE) ### based on test statistics
+            pval <- rowMeans(t(zval.perm) >= x$zval - con$comptol, na.rm=TRUE) ### based on test statistics
          } else {
-            pval <- rowMeans(t(beta.perm) >= c(x$beta) - comp.tol, na.rm=TRUE) ### based on coefficients
+            pval <- rowMeans(t(beta.perm) >= c(x$beta) - con$comptol, na.rm=TRUE) ### based on coefficients
          }
       }
 
       pval[pval > 1] <- 1
 
-      QMp <- mean(QM.perm >= x$QM - comp.tol, na.rm=TRUE)
+      QMp <- mean(QM.perm >= x$QM - con$comptol, na.rm=TRUE)
 
    }
 
@@ -381,16 +393,16 @@ permutest.rma.uni <- function(x, exact=FALSE, iter=1000, permci=FALSE, progbar=T
    ci.lb <- x$ci.lb
    ci.ub <- x$ci.ub
 
-   if ((is.logical(permci) && permci) || is.numeric(permci)) {
+   if (.isTRUE(permci) || is.numeric(permci)) {
 
-      level <- ifelse(x$level > 1, (100-x$level)/100, ifelse(x$level > .5, 1-x$level, x$level))
+      level <- ifelse(x$level == 0, 1, ifelse(x$level >= 1, (100-x$level)/100, ifelse(x$level > .5, 1-x$level, x$level)))
 
       ### check if it is even possible to reject at level
 
       if (1/iter > level / ifelse(con$cialt == "one.sided", 1, 2)) {
 
          permci <- FALSE
-         warning("Cannot obtain ", x$level, "% permutation-based CI; number of permutations (", iter, ") too low.")
+         warning(mstyle$warning("Cannot obtain ", 100*(1-x$level), "% permutation-based CI; number of permutations (", iter, ") too low."))
 
       } else {
 
@@ -400,7 +412,7 @@ permutest.rma.uni <- function(x, exact=FALSE, iter=1000, permci=FALSE, progbar=T
          if (is.numeric(permci)) {
             coefs <- unique(round(permci))
             if (any(coefs > x$p) || any(coefs < 1))
-               stop("Non-existent coefficients specified via 'permci'.")
+               stop(mstyle$stop("Non-existent coefficients specified via 'permci'."))
             permci <- TRUE
          } else {
             coefs <- seq_len(x$p)
@@ -412,7 +424,7 @@ permutest.rma.uni <- function(x, exact=FALSE, iter=1000, permci=FALSE, progbar=T
          for (j in coefs) {
 
             if (progbar)
-               cat("Searching for lower CI bound of coefficient ", j, ": \n", sep="")
+               cat(mstyle$verbose(paste0("Searching for lower CI bound of coefficient ", j, ": \n")))
 
             if (con$cialt == "one.sided") {
                con$alternative <- "greater"
@@ -420,8 +432,8 @@ permutest.rma.uni <- function(x, exact=FALSE, iter=1000, permci=FALSE, progbar=T
                con$alternative <- "two.sided"
             }
 
-            #tmp <- try(uniroot(.permci, interval=c(x$ci.lb[j], coef(x)[j]), extendInt="upX", tol=con$tol, maxiter=con$maxiter, obj=x, j=j, exact=exact, iter=iter, progbar=progbar, comp.tol=comp.tol, level=level, digits=digits, control=con)$root, silent=TRUE)
-            tmp <- try(uniroot(.permci, interval=c(x$ci.lb[j] - con$distfac*(coef(x)[j] - x$ci.lb[j]), coef(x)[j]), extendInt="no", tol=con$tol, maxiter=con$maxiter, obj=x, j=j, exact=exact, iter=iter, progbar=progbar, comp.tol=comp.tol, level=level, digits=digits, control=con)$root, silent=TRUE)
+            #tmp <- try(uniroot(.permci, interval=c(x$ci.lb[j], coef(x)[j]), extendInt="upX", tol=con$tol, maxiter=con$maxiter, obj=x, j=j, exact=exact, iter=iter, progbar=progbar, level=level, digits=digits, control=con)$root, silent=TRUE)
+            tmp <- try(uniroot(.permci, interval=c(x$ci.lb[j] - con$distfac*(coef(x)[j] - x$ci.lb[j]), coef(x)[j]), extendInt="no", tol=con$tol, maxiter=con$maxiter, obj=x, j=j, exact=exact, iter=iter, progbar=progbar, level=level, digits=digits, control=con)$root, silent=TRUE)
 
             if (inherits(tmp, "try-error")) {
                ci.lb[j] <- NA
@@ -430,7 +442,7 @@ permutest.rma.uni <- function(x, exact=FALSE, iter=1000, permci=FALSE, progbar=T
             }
 
             if (progbar)
-               cat("Searching for upper CI bound of coefficient ", j, ": \n", sep="")
+               cat(mstyle$verbose(paste0("Searching for upper CI bound of coefficient ", j, ": \n")))
 
             if (con$cialt == "one.sided") {
                con$alternative <- "less"
@@ -438,8 +450,8 @@ permutest.rma.uni <- function(x, exact=FALSE, iter=1000, permci=FALSE, progbar=T
                con$alternative <- "two.sided"
             }
 
-            #tmp <- try(uniroot(.permci, interval=c(coef(x)[j], x$ci.ub[j]), extendInt="downX", tol=con$tol, maxiter=con$maxiter, obj=x, j=j, exact=exact, iter=iter, progbar=progbar, comp.tol=comp.tol, level=level, digits=digits, control=con)$root, silent=TRUE)
-            tmp <- try(uniroot(.permci, interval=c(coef(x)[j], x$ci.ub[j] + con$distfac*(x$ci.ub[j] - coef(x)[j])), extendInt="no", tol=con$tol, maxiter=con$maxiter, obj=x, j=j, exact=exact, iter=iter, progbar=progbar, comp.tol=comp.tol, level=level, digits=digits, control=con)$root, silent=TRUE)
+            #tmp <- try(uniroot(.permci, interval=c(coef(x)[j], x$ci.ub[j]), extendInt="downX", tol=con$tol, maxiter=con$maxiter, obj=x, j=j, exact=exact, iter=iter, progbar=progbar, level=level, digits=digits, control=con)$root, silent=TRUE)
+            tmp <- try(uniroot(.permci, interval=c(coef(x)[j], x$ci.ub[j] + con$distfac*(x$ci.ub[j] - coef(x)[j])), extendInt="no", tol=con$tol, maxiter=con$maxiter, obj=x, j=j, exact=exact, iter=iter, progbar=progbar, level=level, digits=digits, control=con)$root, silent=TRUE)
 
             if (inherits(tmp, "try-error")) {
                ci.ub[j] <- NA
