@@ -1,15 +1,21 @@
-profile.rma.mv <- function(fitted, sigma2, tau2, rho, gamma2, phi, xlim, ylim, steps=20, startmethod="init", progbar=TRUE, parallel="no", ncpus=1, cl=NULL, plot=TRUE, pch=19, ...) {
+profile.rma.mv <- function(fitted, sigma2, tau2, rho, gamma2, phi,
+   xlim, ylim, steps=20, lltol=1e-06, startmethod="init", progbar=TRUE, parallel="no", ncpus=1, cl=NULL, plot=TRUE, pch=19, cline=FALSE, ...) {
+
+   mstyle <- .get.mstyle("crayon" %in% .packages())
 
    if (!inherits(fitted, "rma.mv"))
-      stop("Argument 'fitted' must be an object of class \"rma.mv\".")
+      stop(mstyle$stop("Argument 'fitted' must be an object of class \"rma.mv\"."))
 
    if (steps < 2)
-      stop("Argument 'steps' must be >= 2.")
+      stop(mstyle$stop("Argument 'steps' must be >= 2."))
 
    x <- fitted
 
    parallel <- match.arg(parallel, c("no", "snow", "multicore"))
 
+   if (parallel == "no" && ncpus > 1)
+      parallel <- "snow"
+
    #########################################################################
 
    ### check if user has specified one of the sigma2, tau2, rho, gamma2, or phi arguments
@@ -23,7 +29,7 @@ profile.rma.mv <- function(fitted, sigma2, tau2, rho, gamma2, phi, xlim, ylim, s
       comps <- ifelse(x$withS, sum(!x$vc.fix$sigma2), 0) + ifelse(x$withG, sum(!x$vc.fix$tau2) + sum(!x$vc.fix$rho), 0) + ifelse(x$withH, sum(!x$vc.fix$gamma2) + sum(!x$vc.fix$phi), 0)
 
       if (comps == 0)
-         stop("No components to profile.")
+         stop(mstyle$stop("No components to profile."))
 
       if (plot) {
          if (dev.cur() == 1) {
@@ -42,9 +48,8 @@ profile.rma.mv <- function(fitted, sigma2, tau2, rho, gamma2, phi, xlim, ylim, s
             mc.vc$sigma2 <- pos
             mc.vc$fitted <- quote(x)
             if (progbar)
-               cat("Profiling sigma2 =", pos, "\n")
+               cat(mstyle$verbose(paste("Profiling sigma2 =", pos, "\n")))
             sav[[j]] <- eval(mc.vc)
-            #sav[[j]] <- profile.rma.mv(x, sigma2=pos, xlim=xlim, ylim=ylim, steps=steps, startmethod=startmethod, progbar=progbar, parallel=parallel, ncpus=ncpus, cl=cl, plot=plot, pch=pch, ...)
          }
       }
 
@@ -56,7 +61,7 @@ profile.rma.mv <- function(fitted, sigma2, tau2, rho, gamma2, phi, xlim, ylim, s
                mc.vc$tau2 <- pos
                mc.vc$fitted <- quote(x)
                if (progbar)
-                  cat("Profiling tau2 =", pos, "\n")
+                  cat(mstyle$verbose(paste("Profiling tau2 =", pos, "\n")))
                sav[[j]] <- eval(mc.vc)
             }
          }
@@ -67,7 +72,7 @@ profile.rma.mv <- function(fitted, sigma2, tau2, rho, gamma2, phi, xlim, ylim, s
                mc.vc$rho <- pos
                mc.vc$fitted <- quote(x)
                if (progbar)
-                  cat("Profiling rho =", pos, "\n")
+                  cat(mstyle$verbose(paste("Profiling rho =", pos, "\n")))
                sav[[j]] <- eval(mc.vc)
             }
          }
@@ -81,7 +86,7 @@ profile.rma.mv <- function(fitted, sigma2, tau2, rho, gamma2, phi, xlim, ylim, s
                mc.vc$gamma2 <- pos
                mc.vc$fitted <- quote(x)
                if (progbar)
-                  cat("Profiling gamma2 =", pos, "\n")
+                  cat(mstyle$verbose(paste("Profiling gamma2 =", pos, "\n")))
                sav[[j]] <- eval(mc.vc)
             }
          }
@@ -92,7 +97,7 @@ profile.rma.mv <- function(fitted, sigma2, tau2, rho, gamma2, phi, xlim, ylim, s
                mc.vc$phi <- pos
                mc.vc$fitted <- quote(x)
                if (progbar)
-                  cat("Profiling phi =", pos, "\n")
+                  cat(mstyle$verbose(paste("Profiling phi =", pos, "\n")))
                sav[[j]] <- eval(mc.vc)
             }
          }
@@ -109,98 +114,98 @@ profile.rma.mv <- function(fitted, sigma2, tau2, rho, gamma2, phi, xlim, ylim, s
    }
 
    #if (missing(sigma2) && missing(tau2) && missing(rho) && missing(gamma2) && missing(phi))
-   #   stop("Must specify one of the arguments 'sigma2', 'tau2', 'rho', 'gamma2', or 'phi'.")
+   #   stop(mstyle$stop("Must specify one of the arguments 'sigma2', 'tau2', 'rho', 'gamma2', or 'phi'."))
 
    ### check if user has specified more than one of these arguments
 
    if (sum(!missing(sigma2), !missing(tau2), !missing(rho), !missing(gamma2), !missing(phi)) > 1L)
-      stop("Must specify only one of the arguments 'sigma2', 'tau2', 'rho', 'gamma2', or 'phi'.")
+      stop(mstyle$stop("Must specify only one of the arguments 'sigma2', 'tau2', 'rho', 'gamma2', or 'phi'."))
 
    ### check if model actually contains (at least one) such a component and that it was actually estimated
    ### note: a component that is not in the model is NA; components that are fixed are TRUE
 
    if (!missing(sigma2) && (all(is.na(x$vc.fix$sigma2)) || all(x$vc.fix$sigma2)))
-      stop("Model does not contain any (estimated) 'sigma2' components.")
+      stop(mstyle$stop("Model does not contain any (estimated) 'sigma2' components."))
 
    if (!missing(tau2) && (all(is.na(x$vc.fix$tau2)) || all(x$vc.fix$tau2)))
-      stop("Model does not contain any (estimated) 'tau2' components.")
+      stop(mstyle$stop("Model does not contain any (estimated) 'tau2' components."))
 
    if (!missing(rho) && c(all(is.na(x$vc.fix$rho)) || all(x$vc.fix$rho)))
-      stop("Model does not contain any (estimated) 'rho' components.")
+      stop(mstyle$stop("Model does not contain any (estimated) 'rho' components."))
 
    if (!missing(gamma2) && (all(is.na(x$vc.fix$gamma2)) || all(x$vc.fix$gamma2)))
-      stop("Model does not contain any (estimated) 'gamma2' components.")
+      stop(mstyle$stop("Model does not contain any (estimated) 'gamma2' components."))
 
    if (!missing(phi) && c(all(is.na(x$vc.fix$phi)) || all(x$vc.fix$phi)))
-      stop("Model does not contain any (estimated) 'phi' components.")
+      stop(mstyle$stop("Model does not contain any (estimated) 'phi' components."))
 
    ### check if user specified more than one sigma2, tau2, rho, gamma2, or rho component
 
    if (!missing(sigma2) && (length(sigma2) > 1L))
-      stop("Can only specify one 'sigma2' component.")
+      stop(mstyle$stop("Can only specify one 'sigma2' component."))
 
    if (!missing(tau2) && (length(tau2) > 1L))
-      stop("Can only specify one 'tau2' component.")
+      stop(mstyle$stop("Can only specify one 'tau2' component."))
 
    if (!missing(rho) && (length(rho) > 1L))
-      stop("Can only specify one 'rho' component.")
+      stop(mstyle$stop("Can only specify one 'rho' component."))
 
    if (!missing(gamma2) && (length(gamma2) > 1L))
-      stop("Can only specify one 'gamma2' component.")
+      stop(mstyle$stop("Can only specify one 'gamma2' component."))
 
    if (!missing(phi) && (length(phi) > 1L))
-      stop("Can only specify one 'phi' component.")
+      stop(mstyle$stop("Can only specify one 'phi' component."))
 
    ### check if user specified a logical
 
    if (!missing(sigma2) && is.logical(sigma2))
-      stop("Must specify the number for the 'sigma2' component.")
+      stop(mstyle$stop("Must specify the number for the 'sigma2' component."))
 
    if (!missing(tau2) && is.logical(tau2))
-      stop("Must specify the number for the 'tau2' component.")
+      stop(mstyle$stop("Must specify the number for the 'tau2' component."))
 
    if (!missing(rho) && is.logical(rho))
-      stop("Must specify the number for the 'rho' component.")
+      stop(mstyle$stop("Must specify the number for the 'rho' component."))
 
    if (!missing(gamma2) && is.logical(gamma2))
-      stop("Must specify the number for the 'gamma2' component.")
+      stop(mstyle$stop("Must specify the number for the 'gamma2' component."))
 
    if (!missing(phi) && is.logical(phi))
-      stop("Must specify the number for the 'phi' component.")
+      stop(mstyle$stop("Must specify the number for the 'phi' component."))
 
    ### check if user specified a component that does not exist
 
    if (!missing(sigma2) && (sigma2 > length(x$vc.fix$sigma2) || sigma2 <= 0))
-      stop("No such 'sigma2' component in the model.")
+      stop(mstyle$stop("No such 'sigma2' component in the model."))
 
    if (!missing(tau2) && (tau2 > length(x$vc.fix$tau2) || tau2 <= 0))
-      stop("No such 'tau2' component in the model.")
+      stop(mstyle$stop("No such 'tau2' component in the model."))
 
    if (!missing(rho) && (rho > length(x$vc.fix$rho) || rho <= 0))
-      stop("No such 'rho' component in the model.")
+      stop(mstyle$stop("No such 'rho' component in the model."))
 
    if (!missing(gamma2) && (gamma2 > length(x$vc.fix$gamma2) || gamma2 <= 0))
-      stop("No such 'gamma2' component in the model.")
+      stop(mstyle$stop("No such 'gamma2' component in the model."))
 
    if (!missing(phi) && (phi > length(x$vc.fix$phi) || phi <= 0))
-      stop("No such 'phi' component in the model.")
+      stop(mstyle$stop("No such 'phi' component in the model."))
 
    ### check if user specified a component that was fixed
 
    if (!missing(sigma2) && x$vc.fix$sigma2[sigma2])
-      stop("Specified 'sigma2' component was fixed.")
+      stop(mstyle$stop("Specified 'sigma2' component was fixed."))
 
    if (!missing(tau2) && x$vc.fix$tau2[tau2])
-      stop("Specified 'tau2' component was fixed.")
+      stop(mstyle$stop("Specified 'tau2' component was fixed."))
 
    if (!missing(rho) && x$vc.fix$rho[rho])
-      stop("Specified 'rho' component was fixed.")
+      stop(mstyle$stop("Specified 'rho' component was fixed."))
 
    if (!missing(gamma2) && x$vc.fix$gamma2[gamma2])
-      stop("Specified 'gamma2' component was fixed.")
+      stop(mstyle$stop("Specified 'gamma2' component was fixed."))
 
    if (!missing(phi) && x$vc.fix$phi[phi])
-      stop("Specified 'phi' component was fixed.")
+      stop(mstyle$stop("Specified 'phi' component was fixed."))
 
    ### if everything is good so far, get value of the variance component and set 'comp'
 
@@ -245,38 +250,102 @@ profile.rma.mv <- function(fitted, sigma2, tau2, rho, gamma2, phi, xlim, ylim, s
    if (missing(xlim)) {
 
       ### if the user has not specified xlim argument, set automatically
-      ### TODO: try something based on CI later
+      ### TODO: maybe try something based on CI later
 
-      if (is.element(comp, c("sigma2", "tau2", "gamma2"))) {
+      if (comp == "sigma2") {
          #vc.lb <- max(.00001, log(vc)) ### old method
          #vc.ub <- max(.00001, exp(vc)) ### old method
          vc.lb <- max( 0, vc/4) ### new method
          vc.ub <- max(.1, vc*4) ### new method
-      } else {
-         vc.lb <- max(-.99999, vc-.5)
-         vc.ub <- min(+.99999, vc+.5)
+      }
+      if (comp == "tau2") {
+         if (is.element(x$struct[1], c("SPEXP","SPGAU","SPLIN","SPRAT","SPSPH","PHYPL","PHYPD"))) {
+            vc.lb <- max( 0, vc/2)
+            vc.ub <- max(.1, vc*2)
+         } else {
+            vc.lb <- max( 0, vc/4)
+            vc.ub <- max(.1, vc*4)
+         }
+      }
+      if (comp == "gamma2") {
+         if (is.element(x$struct[2], c("SPEXP","SPGAU","SPLIN","SPRAT","SPSPH","PHYPL","PHYPD"))) {
+            vc.lb <- max( 0, vc/2)
+            vc.ub <- max(.1, vc*2)
+         } else {
+            vc.lb <- max( 0, vc/4)
+            vc.ub <- max(.1, vc*4)
+         }
+      }
+      if (comp == "rho") {
+         if (x$struct[1] == "CAR") {
+            vc.lb <- max(0, vc-.5)
+            vc.ub <- min(+.99999, vc+.5)
+         }
+         if (is.element(x$struct[1], c("SPEXP","SPGAU","SPLIN","SPRAT","SPSPH","PHYPL","PHYPD"))) {
+            vc.lb <- vc/2
+            vc.ub <- vc*2
+         }
+         if (!is.element(x$struct[1], c("CAR","SPEXP","SPGAU","SPLIN","SPRAT","SPSPH","PHYPL","PHYPD"))) {
+            vc.lb <- max(-.99999, vc-.5)
+            vc.ub <- min(+.99999, vc+.5)
+         }
+      }
+      if (comp == "phi") {
+         if (x$struct[2] == "CAR") {
+            vc.lb <- max(0, vc-.5)
+            vc.ub <- min(+.99999, vc+.5)
+         }
+         if (is.element(x$struct[2], c("SPEXP","SPGAU","SPLIN","SPRAT","SPSPH","PHYPL","PHYPD"))) {
+            vc.lb <- vc/2
+            vc.ub <- vc*2
+         }
+         if (!is.element(x$struct[2], c("CAR","SPEXP","SPGAU","SPLIN","SPRAT","SPSPH","PHYPL","PHYPD"))) {
+            vc.lb <- max(-.99999, vc-.5)
+            vc.ub <- min(+.99999, vc+.5)
+         }
       }
 
       ### if that fails, throw an error
 
       if (is.na(vc.lb) || is.na(vc.ub))
-         stop("Cannot set 'xlim' automatically. Please set this argument manually.")
+         stop(mstyle$stop("Cannot set 'xlim' automatically. Please set this argument manually."))
 
       xlim <- c(vc.lb, vc.ub)
 
    } else {
 
       if (length(xlim) != 2L)
-         stop("Argument 'xlim' should be a vector of length 2.")
+         stop(mstyle$stop("Argument 'xlim' should be a vector of length 2."))
 
       xlim <- sort(xlim)
 
+      if (is.element(comp, c("sigma2", "tau2", "gamma2"))) {
+         if (xlim[1] < 0)
+            stop(mstyle$stop("Lower bound for profiling must be >= 0."))
+      }
+      if (comp == "rho") {
+         if (is.element(x$struct[1], c("CAR","SPEXP","SPGAU","SPLIN","SPRAT","SPSPH","PHYPL","PHYPD")) && xlim[1] < 0)
+            stop(mstyle$stop("Lower bound for profiling must be >= 0."))
+         if (xlim[1] < -1)
+            stop(mstyle$stop("Lower bound for profiling must be >= -1."))
+         if (!is.element(x$struct[1], c("CAR","SPEXP","SPGAU","SPLIN","SPRAT","SPSPH","PHYPL","PHYPD")) && xlim[2] > 1)
+            stop(mstyle$stop("Upper bound for profiling must be <= -1."))
+      }
+      if (comp == "phi") {
+         if (is.element(x$struct[2], c("CAR","SPEXP","SPGAU","SPLIN","SPRAT","SPSPH","PHYPL","PHYPD")) && xlim[1] < 0)
+            stop(mstyle$stop("Lower bound for profiling must be >= 0."))
+         if (xlim[1] < -1)
+            stop(mstyle$stop("Lower bound for profiling must be >= -1."))
+         if (!is.element(x$struct[2], c("CAR","SPEXP","SPGAU","SPLIN","SPRAT","SPSPH","PHYPL","PHYPD")) && xlim[2] > 1)
+            stop(mstyle$stop("Upper bound for profiling must be <= -1."))
+      }
+
    }
 
    vcs <- seq(xlim[1], xlim[2], length=steps)
 
    if (length(vcs) <= 1)
-      stop("Cannot set 'xlim' automatically. Please set this argument manually.")
+      stop(mstyle$stop("Cannot set 'xlim' automatically. Please set this argument manually."))
 
    #return(vcs)
 
@@ -379,11 +448,11 @@ profile.rma.mv <- function(fitted, sigma2, tau2, rho, gamma2, phi, xlim, ylim, s
                x.control[[con.pos.phi.init]] <- res$phi
             }
 
-            res <- try(suppressWarnings(rma.mv(x$yi, x$V, x$W, mods=x$X, intercept=FALSE, random=x$random, struct=x$struct, method=x$method, test=x$test, level=x$level, R=x$R, Rscale=x$Rscale, data=x$mf.r, sigma2=sigma2.arg, tau2=tau2.arg, rho=rho.arg, gamma2=gamma2.arg, phi=phi.arg, control=x.control)), silent=TRUE)
+            res <- try(suppressWarnings(rma.mv(x$yi, V=x$V, W=x$W, mods=x$X, random=x$random, struct=x$struct, intercept=FALSE, data=x$mf.r, method=x$method, test=x$test, level=x$level, R=x$R, Rscale=x$Rscale, sigma2=sigma2.arg, tau2=tau2.arg, rho=rho.arg, gamma2=gamma2.arg, phi=phi.arg, sparse=x$sparse, dist=x$dist, control=x.control)), silent=TRUE)
 
          } else {
 
-            res <- try(suppressWarnings(rma.mv(x$yi, x$V, x$W, mods=x$X, intercept=FALSE, random=x$random, struct=x$struct, method=x$method, test=x$test, level=x$level, R=x$R, Rscale=x$Rscale, data=x$mf.r, sigma2=sigma2.arg, tau2=tau2.arg, rho=rho.arg, gamma2=gamma2.arg, phi=phi.arg, control=x$control)), silent=TRUE)
+            res <- try(suppressWarnings(rma.mv(x$yi, V=x$V, W=x$W, mods=x$X, random=x$random, struct=x$struct, intercept=FALSE, data=x$mf.r, method=x$method, test=x$test, level=x$level, R=x$R, Rscale=x$Rscale, sigma2=sigma2.arg, tau2=tau2.arg, rho=rho.arg, gamma2=gamma2.arg, phi=phi.arg, sparse=x$sparse, dist=x$dist, control=x$control)), silent=TRUE)
 
          }
 
@@ -408,12 +477,12 @@ profile.rma.mv <- function(fitted, sigma2, tau2, rho, gamma2, phi, xlim, ylim, s
    if (parallel=="snow" || parallel == "multicore") {
 
       if (!requireNamespace("parallel", quietly=TRUE))
-         stop("Please install the 'parallel' package for parallel processing.")
+         stop(mstyle$stop("Please install the 'parallel' package for parallel processing."))
 
       ncpus <- as.integer(ncpus)
 
       if (ncpus < 1)
-         stop("Argument 'ncpus' must be >= 1.")
+         stop(mstyle$stop("Argument 'ncpus' must be >= 1."))
 
       if (parallel == "multicore")
          res <- parallel::mclapply(vcs, .profile.rma.mv, obj=x, comp=comp, sigma2.pos=sigma2.pos, tau2.pos=tau2.pos, rho.pos=rho.pos, gamma2.pos=gamma2.pos, phi.pos=phi.pos, mc.cores=ncpus, parallel=parallel, profile=TRUE)
@@ -421,25 +490,29 @@ profile.rma.mv <- function(fitted, sigma2, tau2, rho, gamma2, phi, xlim, ylim, s
       if (parallel == "snow") {
          if (is.null(cl)) {
             cl <- parallel::makePSOCKcluster(ncpus)
-            res <- parallel::parLapply(cl, vcs, .profile.rma.mv, obj=x, comp=comp, sigma2.pos=sigma2.pos, tau2.pos=tau2.pos, rho.pos=rho.pos, gamma2.pos=gamma2.pos, phi.pos=phi.pos, parallel=parallel, profile=TRUE)
-            #res <- parallel::parLapplyLB(cl, vcs, .profile.rma.mv, obj=x, comp=comp, sigma2.pos=sigma2.pos, tau2.pos=tau2.pos, rho.pos=rho.pos, gamma2.pos=gamma2.pos, phi.pos=phi.pos, parallel=parallel, profile=TRUE)
-            #res <- parallel::clusterApply(cl, vcs, .profile.rma.mv, obj=x, comp=comp, sigma2.pos=sigma2.pos, tau2.pos=tau2.pos, rho.pos=rho.pos, gamma2.pos=gamma2.pos, phi.pos=phi.pos, parallel=parallel, profile=TRUE)
-            #res <- parallel::clusterApplyLB(cl, vcs, .profile.rma.mv, obj=x, comp=comp, sigma2.pos=sigma2.pos, tau2.pos=tau2.pos, rho.pos=rho.pos, gamma2.pos=gamma2.pos, phi.pos=phi.pos, parallel=parallel, profile=TRUE)
-            parallel::stopCluster(cl)
-         } else {
-            res <- parallel::parLapply(cl, vcs, .profile.rma.mv, obj=x, comp=comp, sigma2.pos=sigma2.pos, tau2.pos=tau2.pos, rho.pos=rho.pos, gamma2.pos=gamma2.pos, phi.pos=phi.pos, parallel=parallel, profile=TRUE)
+            on.exit(parallel::stopCluster(cl))
          }
+         #res <- parallel::parLapplyLB(cl, vcs, .profile.rma.mv, obj=x, comp=comp, sigma2.pos=sigma2.pos, tau2.pos=tau2.pos, rho.pos=rho.pos, gamma2.pos=gamma2.pos, phi.pos=phi.pos, parallel=parallel, profile=TRUE)
+         #res <- parallel::clusterApply(cl, vcs, .profile.rma.mv, obj=x, comp=comp, sigma2.pos=sigma2.pos, tau2.pos=tau2.pos, rho.pos=rho.pos, gamma2.pos=gamma2.pos, phi.pos=phi.pos, parallel=parallel, profile=TRUE)
+         #res <- parallel::clusterApplyLB(cl, vcs, .profile.rma.mv, obj=x, comp=comp, sigma2.pos=sigma2.pos, tau2.pos=tau2.pos, rho.pos=rho.pos, gamma2.pos=gamma2.pos, phi.pos=phi.pos, parallel=parallel, profile=TRUE)
+         res <- parallel::parLapply(cl, vcs, .profile.rma.mv, obj=x, comp=comp, sigma2.pos=sigma2.pos, tau2.pos=tau2.pos, rho.pos=rho.pos, gamma2.pos=gamma2.pos, phi.pos=phi.pos, parallel=parallel, profile=TRUE)
       }
 
-      lls <- sapply(res, function(z) z$ll)
-      beta  <- do.call("rbind", lapply(res, function(z) t(z$beta)))
-      ci.lb <- do.call("rbind", lapply(res, function(z) t(z$ci.lb)))
-      ci.ub <- do.call("rbind", lapply(res, function(z) t(z$ci.ub)))
+      lls <- sapply(res, function(x) x$ll)
+      beta  <- do.call("rbind", lapply(res, function(x) t(x$beta)))
+      ci.lb <- do.call("rbind", lapply(res, function(x) t(x$ci.lb)))
+      ci.ub <- do.call("rbind", lapply(res, function(x) t(x$ci.ub)))
 
    }
 
    #########################################################################
 
+   if (any(lls >= logLik(x) + lltol, na.rm=TRUE))
+      warning(mstyle$warning("At least one profiled log-likelihood value is larger than the log-likelihood of the fitted model."))
+
+   if (all(is.na(lls)))
+      warning(mstyle$warning("All model fits failed. Cannot draw profile likelihood plot."))
+
    beta  <- data.frame(beta)
    ci.lb <- data.frame(ci.lb)
    ci.ub <- data.frame(ci.ub)
@@ -449,14 +522,18 @@ profile.rma.mv <- function(fitted, sigma2, tau2, rho, gamma2, phi, xlim, ylim, s
 
    if (missing(ylim)) {
 
-      ylim <- range(lls, na.rm=TRUE)
+      if (any(!is.na(lls))) {
+         ylim <- range(lls, na.rm=TRUE)
+      } else {
+         ylim <- rep(logLik(x), 2)
+      }
       ylim[1] <- ylim[1] - .1
       ylim[2] <- ylim[2] + .1
 
    } else {
 
       if (length(ylim) != 2L)
-         stop("Argument 'ylim' should be a vector of length 2.")
+         stop(mstyle$stop("Argument 'ylim' should be a vector of length 2."))
 
       ylim <- sort(ylim)
 
@@ -515,7 +592,7 @@ profile.rma.mv <- function(fitted, sigma2, tau2, rho, gamma2, phi, xlim, ylim, s
    #########################################################################
 
    if (plot)
-      plot(sav, pch=pch, ...)
+      plot(sav, pch=pch, cline=cline, ...)
 
    #########################################################################
 
