@@ -1,55 +1,71 @@
 rma.peto <- function(ai, bi, ci, di, n1i, n2i,
 data, slab, subset,
 add=1/2, to="only0", drop00=TRUE, ### for add/to/drop00, 1st element for escalc(), 2nd for Peto's method
-level=95, digits=4, verbose=FALSE, ...) {
+level=95, digits, verbose=FALSE, ...) {
 
    #########################################################################
 
    ###### setup
 
+   withcrayon <- "crayon" %in% .packages()
+   mstyle <- .get.mstyle(withcrayon)
+
    ### check argument specifications
 
    if (length(add) == 1)
       add <- c(add, 0)
 
    if (length(add) != 2)
-      stop("Argument 'add' should specify one or two values (see 'help(rma.peto)').")
+      stop(mstyle$stop("Argument 'add' should specify one or two values (see 'help(rma.peto)')."))
 
    if (length(to) == 1)
       to <- c(to, "none")
 
    if (length(to) != 2)
-      stop("Argument 'to' should specify one or two values (see 'help(rma.peto)').")
+      stop(mstyle$stop("Argument 'to' should specify one or two values (see 'help(rma.peto)')."))
 
    if (length(drop00) == 1)
       drop00 <- c(drop00, FALSE)
 
    if (length(drop00) != 2)
-      stop("Argument 'drop00' should specify one or two values (see 'help(rma.peto)').")
+      stop(mstyle$stop("Argument 'drop00' should specify one or two values (see 'help(rma.peto)')."))
 
    na.act <- getOption("na.action")
 
    if (!is.element(na.act, c("na.omit", "na.exclude", "na.fail", "na.pass")))
-      stop("Unknown 'na.action' specified under options().")
+      stop(mstyle$stop("Unknown 'na.action' specified under options()."))
 
    if (!is.element(to[1], c("all","only0","if0all","none")))
-      stop("Unknown 'to' argument specified.")
+      stop(mstyle$stop("Unknown 'to' argument specified."))
 
    if (!is.element(to[2], c("all","only0","if0all","none")))
-      stop("Unknown 'to' argument specified.")
+      stop(mstyle$stop("Unknown 'to' argument specified."))
 
    ### get ... argument and check for extra/superfluous arguments
 
    ddd <- list(...)
 
-   .chkdots(ddd, c("outlist"))
+   .chkdots(ddd, c("outlist", "time"))
+
+   ### handle 'time' argument from ...
+
+   if (.isTRUE(ddd$time))
+      time.start <- proc.time()
 
    measure <- "PETO" ### set measure here so that it can be added below
 
+   ### set defaults for digits
+
+   if (missing(digits)) {
+      digits <- .set.digits(dmiss=TRUE)
+   } else {
+      digits <- .set.digits(digits, dmiss=FALSE)
+   }
+
    #########################################################################
 
    if (verbose)
-      message("Extracting data and computing yi/vi values ...")
+      message(mstyle$message("\nExtracting data and computing yi/vi values ..."))
 
    ### check if data argument has been specified
 
@@ -80,23 +96,25 @@ level=95, digits=4, verbose=FALSE, ...) {
    mf.di  <- mf[[match("di",  names(mf))]]
    mf.n1i <- mf[[match("n1i", names(mf))]]
    mf.n2i <- mf[[match("n2i", names(mf))]]
-   ai     <- eval(mf.ai,  data, enclos=sys.frame(sys.parent()))
-   bi     <- eval(mf.bi,  data, enclos=sys.frame(sys.parent()))
-   ci     <- eval(mf.ci,  data, enclos=sys.frame(sys.parent()))
-   di     <- eval(mf.di,  data, enclos=sys.frame(sys.parent()))
-   n1i    <- eval(mf.n1i, data, enclos=sys.frame(sys.parent()))
-   n2i    <- eval(mf.n2i, data, enclos=sys.frame(sys.parent()))
+   ai  <- eval(mf.ai,  data, enclos=sys.frame(sys.parent()))
+   bi  <- eval(mf.bi,  data, enclos=sys.frame(sys.parent()))
+   ci  <- eval(mf.ci,  data, enclos=sys.frame(sys.parent()))
+   di  <- eval(mf.di,  data, enclos=sys.frame(sys.parent()))
+   n1i <- eval(mf.n1i, data, enclos=sys.frame(sys.parent()))
+   n2i <- eval(mf.n2i, data, enclos=sys.frame(sys.parent()))
    if (is.null(bi)) bi <- n1i - ai
    if (is.null(di)) di <- n2i - ci
-   ni     <- ai + bi + ci + di
+   ni <- ai + bi + ci + di
+
+   k <- length(ai) ### number of outcomes before subsetting
+   k.all <- k
 
-   k   <- length(ai)
    ids <- seq_len(k)
 
    ### generate study labels if none are specified
 
    if (verbose)
-      message("Generating/extracting study labels ...")
+      message(mstyle$message("Generating/extracting study labels ..."))
 
    if (is.null(slab)) {
 
@@ -106,10 +124,10 @@ level=95, digits=4, verbose=FALSE, ...) {
    } else {
 
       if (anyNA(slab))
-         stop("NAs in study labels.")
+         stop(mstyle$stop("NAs in study labels."))
 
       if (length(slab) != k)
-         stop("Study labels not of same length as data.")
+         stop(mstyle$stop("Study labels not of same length as data."))
 
       slab.null <- FALSE
 
@@ -120,7 +138,7 @@ level=95, digits=4, verbose=FALSE, ...) {
    if (!is.null(subset)) {
 
       if (verbose)
-         message("Subsetting ...")
+         message(mstyle$message("Subsetting ..."))
 
       ai   <- ai[subset]
       bi   <- bi[subset]
@@ -175,7 +193,7 @@ level=95, digits=4, verbose=FALSE, ...) {
    if (any(has.na)) {
 
       if (verbose)
-         message("Handling NAs in table data ...")
+         message(mstyle$message("Handling NAs in table data ..."))
 
       if (na.act == "na.omit" || na.act == "na.exclude" || na.act == "na.pass") {
          ai   <- ai[not.na]
@@ -183,18 +201,18 @@ level=95, digits=4, verbose=FALSE, ...) {
          ci   <- ci[not.na]
          di   <- di[not.na]
          k    <- length(ai)
-         warning("Tables with NAs omitted from model fitting.")
+         warning(mstyle$warning("Tables with NAs omitted from model fitting."))
       }
 
       if (na.act == "na.fail")
-         stop("Missing values in tables.")
+         stop(mstyle$stop("Missing values in tables."))
 
    }
 
    ### at least one study left?
 
    if (k < 1)
-      stop("Processing terminated since k = 0.")
+      stop(mstyle$stop("Processing terminated since k = 0."))
 
    ### check for NAs in yi/vi and act accordingly
 
@@ -204,14 +222,14 @@ level=95, digits=4, verbose=FALSE, ...) {
    if (any(yivi.na)) {
 
       if (verbose)
-         message("Handling NAs in yi/vi ...")
+         message(mstyle$message("Handling NAs in yi/vi ..."))
 
       if (na.act == "na.omit" || na.act == "na.exclude" || na.act == "na.pass") {
 
          yi <- yi[not.na.yivi]
          vi <- vi[not.na.yivi]
          ni <- ni[not.na.yivi]
-         warning("Some yi/vi values are NA.")
+         warning(mstyle$warning("Some yi/vi values are NA."))
 
          attr(yi, "measure") <- measure ### add measure attribute back
          attr(yi, "ni")      <- ni      ### add ni attribute back
@@ -219,7 +237,7 @@ level=95, digits=4, verbose=FALSE, ...) {
       }
 
       if (na.act == "na.fail")
-         stop("Missing yi/vi values.")
+         stop(mstyle$stop("Missing yi/vi values."))
 
    }
 
@@ -275,12 +293,12 @@ level=95, digits=4, verbose=FALSE, ...) {
 
    #########################################################################
 
-   level <- ifelse(level > 1, (100-level)/100, ifelse(level > .5, 1-level, level))
+   level <- ifelse(level == 0, 1, ifelse(level >= 1, (100-level)/100, ifelse(level > .5, 1-level, level)))
 
    ###### model fitting, test statistics, and confidence intervals
 
    if (verbose)
-      message("Model fitting ...")
+      message(mstyle$message("Model fitting ..."))
 
    xt <- ai + ci ### frequency of outcome1 in both groups combined
    yt <- bi + di ### frequency of outcome2 in both groups combined
@@ -290,7 +308,7 @@ level=95, digits=4, verbose=FALSE, ...) {
    sumVi <- sum(Vi)
 
    if (sumVi == 0L) ### sumVi = 0 when xt or yt = 0 in *all* tables
-      stop("One of the two outcomes never occurred in any of the tables. Peto's method cannot be used.")
+      stop(mstyle$stop("One of the two outcomes never occurred in any of the tables. Peto's method cannot be used."))
 
    beta  <- sum(ai - Ei) / sumVi
    se    <- sqrt(1/sumVi)
@@ -307,7 +325,7 @@ level=95, digits=4, verbose=FALSE, ...) {
    ### heterogeneity test (Peto's method)
 
    if (verbose)
-      message("Heterogeneity testing ...")
+      message(mstyle$message("Heterogeneity testing ..."))
 
    k.pos <- sum(Vi > 0) ### number of tables with positive sampling variance
    Vi[Vi == 0] <- NA    ### set 0 sampling variances to NA
@@ -331,7 +349,7 @@ level=95, digits=4, verbose=FALSE, ...) {
    ###### fit statistics
 
    if (verbose)
-      message("Computing fit statistics and log likelihood ...")
+      message(mstyle$message("Computing fit statistics and log likelihood ..."))
 
    ll.ML     <- -1/2 * (k.yi)   * log(2*base::pi)                   - 1/2 * sum(log(vi))                      - 1/2 * RSS
    ll.REML   <- -1/2 * (k.yi-1) * log(2*base::pi) + 1/2 * log(k.yi) - 1/2 * sum(log(vi)) - 1/2 * log(sum(wi)) - 1/2 * RSS
@@ -353,7 +371,7 @@ level=95, digits=4, verbose=FALSE, ...) {
    ###### prepare output
 
    if (verbose)
-      message("Preparing output ...")
+      message(mstyle$message("Preparing output ..."))
 
    parms     <- 1
    p         <- 1
@@ -373,17 +391,26 @@ level=95, digits=4, verbose=FALSE, ...) {
 
       res <- list(b=beta, beta=beta, se=se, zval=zval, pval=pval, ci.lb=ci.lb, ci.ub=ci.ub, vb=vb,
                   tau2=tau2,
-                  k=k, k.f=k.f, k.yi=k.yi, k.pos=k.pos, k.eff=k.eff, p=p, parms=parms,
+                  k=k, k.f=k.f, k.yi=k.yi, k.pos=k.pos, k.eff=k.eff, k.all=k.all, p=p, parms=parms,
                   QE=QE, QEp=QEp, I2=I2, H2=H2,
                   int.only=int.only,
                   yi=yi, vi=vi, yi.f=yi.f, vi.f=vi.f, X.f=X.f, ai=ai, bi=bi, ci=ci, di=di, ai.f=ai.f, bi.f=bi.f, ci.f=ci.f, di.f=di.f, ni=ni, ni.f=ni.f,
-                  ids=ids, not.na=not.na, not.na.yivi=not.na.yivi, slab=slab, slab.null=slab.null,
+                  ids=ids, not.na=not.na, subset=subset, not.na.yivi=not.na.yivi, slab=slab, slab.null=slab.null,
                   measure=measure, method=method, weighted=weighted, test=test, dfs=dfs, intercept=intercept, digits=digits, level=level,
                   add=add, to=to, drop00=drop00,
-                  fit.stats=fit.stats, call=mf)
+                  fit.stats=fit.stats, formula.yi=NULL, formula.mods=NULL, version=packageVersion("metafor"), call=mf)
 
    }
 
+   if (.isTRUE(ddd$time)) {
+      time.end <- proc.time()
+      res$time <- unname(time.end - time.start)[3]
+      .print.time(res$time)
+   }
+
+   if (verbose || .isTRUE(ddd$time))
+      cat("\n")
+
    if (!is.null(ddd$outlist)) {
       if (ddd$outlist == "minimal") {
          res <- list(b=beta, beta=beta, se=se, zval=zval, pval=pval, ci.lb=ci.lb, ci.ub=ci.ub, vb=vb, digits=digits, k=k, k.pos=k.pos, k.eff=k.eff, p=p, parms=parms, fit.stats=fit.stats, QE=QE, QEp=QEp)
