@@ -2,6 +2,8 @@
 
 context("Checking misc: robust() function")
 
+source("tolerances.r") # read in tolerances
+
 test_that("robust() works correctly for rma().", {
 
    data(dat.bcg, package="metafor")
@@ -9,14 +11,14 @@ test_that("robust() works correctly for rma().", {
    res <- rma(yi, vi, data=dat)
 
    sav <- robust(res, cluster=dat$trial)
-   expect_equivalent(round(c(vcov(sav)), 6), 0.032106)
+   expect_equivalent(c(vcov(sav)), 0.032106, tolerance=.tol[["var"]])
 
    sav <- robust(res, cluster=dat$trial, adjust=FALSE)
-   expect_equivalent(round(c(vcov(sav)), 6), 0.029636)
+   expect_equivalent(c(vcov(sav)), 0.029636, tolerance=.tol[["var"]])
 
    res <- rma(yi, vi, weights=1, data=dat)
    sav <- robust(res, cluster=dat$trial)
-   expect_equivalent(round(c(vcov(sav)), 6), 0.037028)
+   expect_equivalent(c(vcov(sav)), 0.037028, tolerance=.tol[["var"]])
 
 })
 
@@ -27,13 +29,13 @@ test_that("robust() works correctly for rma.mv().", {
    res <- rma.mv(yi, vi, random = ~ 1 | trial, data=dat)
 
    sav <- robust(res, cluster=dat$trial)
-   expect_equivalent(round(c(vcov(sav)), 6), 0.032106)
+   expect_equivalent(c(vcov(sav)), 0.032106, tolerance=.tol[["var"]])
 
    sav <- robust(res, cluster=dat$trial, adjust=FALSE)
-   expect_equivalent(round(c(vcov(sav)), 6), 0.029636)
+   expect_equivalent(c(vcov(sav)), 0.029636, tolerance=.tol[["var"]])
 
    res <- rma.mv(yi, vi, W=1, random = ~ 1 | trial, data=dat)
    sav <- robust(res, cluster=dat$trial)
-   expect_equivalent(round(c(vcov(sav)), 6), 0.037028)
+   expect_equivalent(c(vcov(sav)), 0.037028, tolerance=.tol[["var"]])
 
 })
